@@ -727,17 +727,19 @@ eog_image_set_icc_data (EogImage *img, EogMetadataReader *md_reader)
 }
 #endif
 
-#ifdef HAVE_EXIF
 static void
 eog_image_set_orientation (EogImage *img)
 {
 	EogImagePrivate *priv;
+#ifdef HAVE_EXIF
 	ExifData* exif;
+#endif
 
 	g_return_if_fail (EOG_IS_IMAGE (img));
 
 	priv = img->priv;
 
+#ifdef HAVE_EXIF
 	exif = (ExifData*) eog_image_get_exif_info (img);
 
 	if (exif != NULL) {
@@ -749,10 +751,28 @@ eog_image_set_orientation (EogImage *img)
 		if (entry && entry->data != NULL) {
 			priv->orientation = exif_get_short (entry->data, o);
 		}
-	}
 
-	/* exif_data_unref handles NULL values like g_free */
-	exif_data_unref (exif);
+		exif_data_unref (exif);
+	} else
+#endif
+	{
+		GdkPixbuf *pbuf;
+
+		pbuf = eog_image_get_pixbuf (img);
+
+		if (pbuf) {
+			const gchar *o_str;
+
+			o_str = gdk_pixbuf_get_option (pbuf, "orientation");
+			if (o_str) {
+				short t = (short) g_ascii_strtoll (o_str,
+								   NULL, 10);
+				if (t >= 0 && t < 9)
+					priv->orientation = t;
+			}
+			g_object_unref (pbuf);
+		}
+	}
 
 	if (priv->orientation > 4 &&
 	    priv->orientation < 9) {
@@ -801,7 +821,6 @@ eog_image_autorotate (EogImage *img)
 	/* Schedule auto orientation */
 	img->priv->autorotate = TRUE;
 }
-#endif
 
 #ifdef HAVE_EXEMPI
 static void
@@ -897,6 +916,7 @@ eog_image_real_load (EogImage *img,
 	gboolean failed = FALSE;
 	gboolean first_run = TRUE;
 	gboolean set_metadata = TRUE;
+	gboolean use_rsvg = FALSE;
 	gboolean read_image_data = (data2read & EOG_IMAGE_DATA_IMAGE);
 	gboolean read_only_dimension = (data2read & EOG_IMAGE_DATA_DIMENSION) &&
 				  ((data2read ^ EOG_IMAGE_DATA_DIMENSION) == 0);
@@ -967,39 +987,43 @@ eog_image_real_load (EogImage *img,
 			gchar *file_path;
 			/* Keep the object for rendering */
 			priv->svg = rsvg_handle_new ();
+			use_rsvg = (priv->svg != NULL);
 			file_path = g_file_get_path (priv->file);
 			rsvg_handle_set_base_uri (priv->svg, file_path);
 			g_free (file_path);
 		}
 #endif
-		loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
 
-		if (error && *error) {
-			g_error_free (*error);
-			*error = NULL;
+		if (!use_rsvg) {
+			loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
 
-			loader = gdk_pixbuf_loader_new ();
-		} else {
-			/* The mimetype-based loader should know the
-			 * format here already. */
-			checked_threadsafety = check_loader_threadsafety (loader, &priv->threadsafe_format);
-		}
+			if (error && *error) {
+				g_error_free (*error);
+				*error = NULL;
+
+				loader = gdk_pixbuf_loader_new ();
+			} else {
+				/* The mimetype-based loader should know the
+				 * format here already. */
+				checked_threadsafety = check_loader_threadsafety (loader, &priv->threadsafe_format);
+			}
 
 		/* This is used to detect non-threadsafe loaders and disable
  		 * any possible asyncronous task that could bring deadlocks
  		 * to image loading process. */
-		if (!checked_threadsafety)
-			g_signal_connect (loader,
+			if (!checked_threadsafety)
+				g_signal_connect (loader,
 					  "size-prepared",
 					  G_CALLBACK (eog_image_pre_size_prepared),
 					  img);
 
-		g_signal_connect_object (G_OBJECT (loader),
+			g_signal_connect_object (G_OBJECT (loader),
 					 "size-prepared",
 					 G_CALLBACK (eog_image_size_prepared),
 					 img,
 					 0);
-        }
+		}
+	}
 	g_free (mime_type);
 
 	while (!priv->cancel_loading) {
@@ -1024,17 +1048,23 @@ eog_image_real_load (EogImage *img,
 		}
 
 		if ((read_image_data || read_only_dimension)) {
+#ifdef HAVE_RSVG
+			if (use_rsvg) {
+				gboolean res;
+
+				res = rsvg_handle_write (priv->svg, buffer,
+							 bytes_read, error);
+
+				if (G_UNLIKELY (!res)) {
+					failed = TRUE;
+					break;
+				}
+			} else
+#endif
 			if (!gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
 				failed = TRUE;
 				break;
 			}
-#ifdef HAVE_RSVG
-			if (eog_image_is_svg (img) &&
-			    !rsvg_handle_write (priv->svg, buffer, bytes_read, error)) {
-				failed = TRUE;
-				break;
-			}
-#endif
 		}
 
 		bytes_read_total += bytes_read;
@@ -1095,6 +1125,11 @@ eog_image_real_load (EogImage *img,
 	}
 
 	if (read_image_data || read_only_dimension) {
+#ifdef HAVE_RSVG
+		if (use_rsvg) {
+			rsvg_handle_close (priv->svg, error);
+		} else
+#endif
 		if (failed) {
 			gdk_pixbuf_loader_close (loader, NULL);
 		} else if (!gdk_pixbuf_loader_close (loader, error)) {
@@ -1104,10 +1139,6 @@ eog_image_real_load (EogImage *img,
 				g_clear_error (error);
 			}
 	        }
-#ifdef HAVE_RSVG
-		if (eog_image_is_svg (img))
-			rsvg_handle_close (priv->svg, error);
-#endif
         }
 
 	g_free (buffer);
@@ -1131,6 +1162,13 @@ eog_image_real_load (EogImage *img,
 			g_object_unref (priv->image);
 		}
 
+#ifdef HAVE_RSVG
+		if (use_rsvg) {
+			priv->image = rsvg_handle_get_pixbuf (priv->svg);
+		} else
+#endif
+		{
+
 		priv->anim = gdk_pixbuf_loader_get_animation (loader);
 
 		if (gdk_pixbuf_animation_is_static_image (priv->anim)) {
@@ -1141,13 +1179,21 @@ eog_image_real_load (EogImage *img,
 			priv->image = gdk_pixbuf_animation_iter_get_pixbuf (priv->anim_iter);
 		}
 
+		}
+
 		if (G_LIKELY (priv->image != NULL)) {
-			g_object_ref (priv->image);
+			if (!use_rsvg)
+				g_object_ref (priv->image);
 
 			priv->width = gdk_pixbuf_get_width (priv->image);
 			priv->height = gdk_pixbuf_get_height (priv->image);
 
-			format = gdk_pixbuf_loader_get_format (loader);
+			if (use_rsvg) {
+				format = NULL;
+				priv->file_type = g_strdup ("svg");
+			} else {
+				format = gdk_pixbuf_loader_get_format (loader);
+			}
 
 			if (format != NULL) {
 				priv->file_type = gdk_pixbuf_format_get_name (format);
@@ -1155,6 +1201,10 @@ eog_image_real_load (EogImage *img,
 
 			priv->file_is_changed = FALSE;
 
+			/* Set orientation again for safety, eg. if we don't
+			 * have Exif data or HAVE_EXIF is undefined. */
+			eog_image_set_orientation (img);
+
 			/* If it's non-threadsafe loader, then trigger window
  			 * showing in the end of the process. */
 			if (!priv->threadsafe_format)
@@ -1255,16 +1305,17 @@ eog_image_load (EogImage *img, EogImageData data2read, EogJob *job, GError **err
 
 	success = eog_image_real_load (img, data2read, job, error);
 
-#ifdef HAVE_EXIF
+
 	/* Check that the metadata was loaded at least once before
 	 * trying to autorotate. Also only an image load job should try to
 	 * autorotate an image. */
-	if (priv->autorotate && 
-	    priv->metadata_status == EOG_IMAGE_METADATA_READY &&
+	if (priv->autorotate &&
+#ifdef HAVE_EXIF
+	    priv->metadata_status != EOG_IMAGE_METADATA_NOT_READ &&
+#endif
 	    data2read & EOG_IMAGE_DATA_IMAGE) {
 		eog_image_real_autorotate (img);
 	}
-#endif
 
 	if (success && eog_image_needs_transformation (img)) {
 		success = eog_image_apply_transformations (img, error);
@@ -1936,14 +1987,14 @@ eog_image_get_exif_info (EogImage *img)
 gpointer
 eog_image_get_xmp_info (EogImage *img)
 {
-	EogImagePrivate *priv;
  	gpointer data = NULL;
 
  	g_return_val_if_fail (EOG_IS_IMAGE (img), NULL);
 
+#ifdef HAVE_EXEMPI
+	EogImagePrivate *priv;
  	priv = img->priv;
 
-#ifdef HAVE_EXEMPI
  	g_mutex_lock (priv->status_mutex);
  	data = (gpointer) xmp_copy (priv->xmp);
  	g_mutex_unlock (priv->status_mutex);
